import 'dart:ffi';
import 'dart:io';
import 'package:meta/meta.dart';
import 'mdb_consts.dart';
import 'typedefs.dart';

class LMDBFunctions {
  final MdbEnvCreate mdbEnvCreate;

  final MdbEnvSetMaxdbs mdbEnvSetMaxdbs;

  final MdbStrError mdbStrError;

  final MdbEnvOpen mdbEnvOpen;

  final MdbTxnBegin mdbTxnBegin;

  final MdbDbiOpen mdbDbiOpen;

  final MdbGet mdbGet;

  final MdbPut mdbPut;

  final MdbDel mdbDel;

  final MdbCursorOpen mdbCursorOpen;

  final MdbCursorGet mdbCursorGet;

  final MdbCursorClose mdbCursorClose;

  final MdbTxnCommit mdbTxnCommit;

  final MdbEnvSync mdbEnvSync;

  final MdbEnvClose mdbEnvClose;

  LMDBFunctions._({
    @required this.mdbTxnBegin,
    @required this.mdbEnvSetMaxdbs,
    @required this.mdbEnvCreate,
    @required this.mdbEnvOpen,
    @required this.mdbStrError,
    @required this.mdbCursorOpen,
    @required this.mdbCursorGet,
    @required this.mdbDbiOpen,
    @required this.mdbPut,
    @required this.mdbGet,
    @required this.mdbDel,
    @required this.mdbTxnCommit,
    @required this.mdbEnvSync,
    @required this.mdbCursorClose,
    @required this.mdbEnvClose,
  })  : assert(mdbTxnCommit != null),
        assert(mdbEnvOpen != null),
        assert(mdbEnvSync != null),
        assert(mdbEnvCreate != null),
        assert(mdbEnvSetMaxdbs != null),
        assert(mdbStrError != null),
        assert(mdbTxnBegin != null),
        assert(mdbDbiOpen != null),
        assert(mdbPut != null),
        assert(mdbGet != null),
        assert(mdbDel != null),
        assert(mdbCursorOpen != null),
        assert(mdbEnvClose != null);

  static DynamicLibrary load() {
    if (Platform.isWindows) {
      return DynamicLibrary.open('liblmdb.dll');
    } else if (Platform.isLinux || Platform.isLinux || Platform.isFuchsia) {
      return DynamicLibrary.open('liblmdb.so');
    } else {
      return DynamicLibrary.process();
    }
  }

  static LMDBFunctions _instance;

  factory LMDBFunctions() {
    if (_instance == null) {
      final lmdb = load();
      _instance = LMDBFunctions._(
        mdbEnvSync: lmdb
            .lookup<NativeFunction<EnvSync>>(MDBConsts.ENV_SYNC)
            .asFunction(),
        mdbEnvClose: lmdb
            .lookup<NativeFunction<EnvClose>>(MDBConsts.ENV_CLOSE)
            .asFunction(),
        mdbEnvCreate: lmdb
            .lookup<NativeFunction<EnvCreate>>(MDBConsts.CREATE)
            .asFunction(),
        mdbEnvSetMaxdbs: lmdb
            .lookup<NativeFunction<SetMaxDBS>>(MDBConsts.SET_MAX_DBS)
            .asFunction(),
        mdbEnvOpen: lmdb
            .lookup<NativeFunction<EnvOpen>>(MDBConsts.ENV_OPEN)
            .asFunction(),
        mdbStrError: lmdb
            .lookup<NativeFunction<StrError>>(MDBConsts.STR_ERROR)
            .asFunction(),
        mdbTxnBegin: lmdb
            .lookup<NativeFunction<TxtBegin>>(MDBConsts.TXT_BEGIN)
            .asFunction(),
        mdbDbiOpen: lmdb
            .lookup<NativeFunction<DbiOpen>>(MDBConsts.DBI_OPEN)
            .asFunction(),
        mdbPut: lmdb
            .lookup<NativeFunction<MdbPutP>>(MDBConsts.MDB_PUT)
            .asFunction(),
        mdbGet: lmdb
            .lookup<NativeFunction<MdbGetP>>(MDBConsts.MDB_GET)
            .asFunction(),
        mdbDel: lmdb
            .lookup<NativeFunction<MdbGetP>>(MDBConsts.MDB_DEL)
            .asFunction(),
        mdbCursorOpen: lmdb
            .lookup<NativeFunction<CursorOpen>>(MDBConsts.CURSOR_OPEN)
            .asFunction(),
        mdbCursorGet: lmdb
            .lookup<NativeFunction<CursorGet>>(MDBConsts.CURSOR_GET)
            .asFunction(),
        mdbCursorClose: lmdb
            .lookup<NativeFunction<CursorClose>>(MDBConsts.CURSOR_CLOSE)
            .asFunction(),
        mdbTxnCommit: lmdb
            .lookup<NativeFunction<TxtCommit>>(MDBConsts.TXN_COMMIT)
            .asFunction(),
      );
    }

    return _instance;
  }
}
